;+**********************************************************************
; SCCS id: %W% %U% %G% 
;***********************************************************************
; PROJECT:   Magnetic holes
;
; COMPONENT: Low B
;
; LANGUAGE:  IDL
;
; NAME:             HEOMAT
;
; CALLING_SEQUENCE: matrix=HEOMAT(time,[posvector],in='GEO',out='HGRTN')
;
; PURPOSE: Transform from geocentric to heliocentric systems.
;
; DESCRIPTION: Use formulae for the mean position
;         of the Sun and the solar axis to calculate
;         transformation matrix for given time.
; SOURCES:  K.R.Lang, Astrophys.Formulae 2nd ed, Springer,1980
;         P.K.Seidelmann (edt), Explanatory Supplement
;           to the Astronomical Almanac, 2nd ed.,CA,1992
;         M.Hapgood, Plan.Space Sci.,40,711,1992 (Hapgood1)
;         M.Hapgood, Plan.Space Sci.,45,1047,1997
;         M.Hapgood, Ann.Geophys.,13,713,1995
;         L.Burlaga, Space Sci.Rev,39,255,1984
;
; ARGUMENTS:
;    intime    DBLARR(tpoints)  tj Julian century from J2000.0
;
;  OPTIONAL:
;    posvector DBLARR(tpoints,3)  position of SC in possys coord [km]
;                      if not provided position of earth is assumed.
;                      If no possys keyword is provided insys is assumed.
;
;    KEYWORDS: insys/outsys/possys  STRING  one of the following (Hapgood1):
;        'GSE' geocentric solar ecliptic [default insys]
;              X=E-S-line Z=ecliptic N
;        'HAE' heliocentric aries ecliptic
;              X=Aries Z=ecliptic North
;        'HEE' heliocentric earth ecliptic
;              X=S-E-line Z=ecliptic North
;        'HSE' heliographic solar equatorial (Burlaga,NSSDC)
;              X=solar asc.node Z=solar pole
;              Burlaga seems to use a fixed axis at
;              solar asc.node 1900.0 while NSSDC uses
;              the true ascending node. We use true asc.node.
;        'HGC' Carrington system (=NSSDC heliographic HG)
;              X=solar ecliptic asc.node meridian on 1 Jan 1854,12UT 
;             (JD2398220.0=MJD1780.5)
;                +days/25.38 Z=solar pole
;        'HEEQ' heliocentric earth equatorial
;              X=solar central meridian at solar equator
;              Z=solar pole
;        'HGRTN' heliographic RTN [default outsys]
;              = Burlaga HG 
;              this is Hapgoods HEEQ transformed to RadialTangentialNormal
;              for a given input vector
;        'HERTN' solar ecliptic RTN
;              RadialTangentialNormal with respect to ecliptic North  
;              for a given input vector
;              (This is the Helios SSE system with changed sign for x and z,
;              assuming that the Helios orbit is in the ecliptic.)
;        'GSM' geocentric solar magnetospheric
;              X=E-S-line Z=earth dipole axis on GSE YZ
;        'SMA' geocentric solar magnetic
;              Y=earth dipole X E-S-line Z=Earth dipole
;        'MAG' geocentric geomagnetic
;              Z=dipole axis Y=dipole longitude +90deg East
;        'GEI' geocentric equatorial inertial
;              X=Aries Z=GEO North
;        'GEO' geographic
;         lon  get back geocentric solar ecliptic longitude
;
;  RETURNS:  matrix FLTARR(3,3,tpoints) conversion from GSE to RTN
;                   multiply from left: vec=vec#GEOMAT([0])
;
; FUNCTIONS USED: MARRMULO,GPOLROT,VECPRD
;
; RESTRICTIONS: Routine returns array of rotation matrices.
;       Note that to transform velocity vectors into inertial
;       systems you have to do the velocity transformation
;       separately (see GEOVEC).
; TEST DATA: from Mike Hapgood:
;From mah@jsoc1.bnsc.rl.ac.uk Mon Sep  8 16:23 BST 1997
;
;I have extended test 5 to include the other coordinate systems - but only
;with the rotations as if it were a velocity or field vector. If the vector 
;were a position vector an offset of origin would also be applied. I enclose
;my results below,
;
;Cheers,
;Mike
;
;Time is 1996-08-28T16:46:00Z = 1996.2411646d
;
; Rotation angles are:
; 
;    1. Greenwich Sideral Time       =  228.6813
;   2. Solar Longitude              =  155.6438
;    3. Obliquity of the ecliptic    =   23.4394
;    4. Rotation from GSE to GSM     =   21.6079
;    5. Dipole tilt (GSM to SM)      =   20.0904
;    6. Latitude of the dipole pole  =   79.3514
;    7. Longitude of the dipole pole =  288.4819
;    8. Ascending node of solar equ. =   75.7137
;    9. Inclination of solar equator =    7.2500
;   10. Solar longitude of Earth     =  259.8506
;   11. Solar latitude of Earth      =    7.1377
;   12. XY inversion                 =  180.0000
; Clustran Test 5
;  GSE: X =  4.03300, Y =  5.12200, Z =  3.39100, R =  7.34839
;  GSM: X =  4.03300, Y =  6.01080, Z =  1.26651, R =  7.34839
;   SM: X =  3.35255, Y =  6.01080, Z =  2.57479, R =  7.34839
; GEOC: X =  6.90274, Y = -1.63624, Z =  1.91669, R =  7.34839
;  GEI: X = -5.78641, Y = -4.10397, Z =  1.91669, R =  7.34839
;  MAG: X =  3.32147, Y =  6.02803, Z =  2.57479, R =  7.34839
; GSEQ: X =  4.03300, Y =  5.19614, Z =  3.27626, R =  7.34839
; HEEQ: X = -4.42309, Y = -5.05705, Z =  2.97680, R =  7.34839
;  HEE: X = -4.03300, Y = -5.12200, Z =  3.39100, R =  7.34839
;  HAE: X = -5.78641, Y = -3.00290, Z =  3.39100, R =  7.34839
;
; ---------------------------------------------------------------------
; Copyright (c) 1996 W.J.M.Fraenz
;
; Astronomy Unit, Queen Mary and Westfield College, London, UK.
; This software is distributed WITHOUT ANY WARRANTY, and without even the
; implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
; complete copy of this copyright message with the software.
; ---------------------------------------------------------------------
; HISTORY:  created   2000    by Markus Fraenz (MF) @qmw
;     31/08/00  HSE,HGC,HERTN,possys added for Helios MF
;     05/12/01  minor corrections MF
;
;-***********************************************************************


 FUNCTION HEOMAT,tj,posvector,insys=insys,outsys=outsys,lon=lon,$
     possys=possys,debug=debug

  debug=KEYWORD_SET(debug)
  
  nelin=N_ELEMENTS(tj)
  nelvec=N_ELEMENTS(posvector)
  IF nelvec GT 0 THEN BEGIN
   IF nelvec NE 3*nelin THEN MESSAGE,'Incorrect number of elements: posvector'
  ENDIF
  IF NOT KEYWORD_SET(outsys) THEN outsys='HGRTN' ELSE outsys=STRUPCASE(outsys)

  IF NOT KEYWORD_SET(insys) THEN insys='GSE' ELSE insys=STRUPCASE(insys)

  hset=(STRPOS(insys,'H') GE 0) OR (STRPOS(outsys,'H') GE 0)
  smset=(STRPOS(insys,'SMA') GE 0) OR (STRPOS(outsys,'SMA') GE 0)
  geoin=(STRPOS(insys,'H') LT 0)


  pi2=!dpi/2.0d
  ddtor=!dpi/180.0d
  dj=tj*36525d
  jd2000= 2451545.0d



  IF insys EQ outsys THEN BEGIN
   unity=DBLARR(3,3,nelin)
   FOR i=0,2 DO unity(i,i,*)=1.0d
   RETURN,unity
  ENDIF

  ; Earth longitude:
  dlm= ((100.4664568d)+((35999.3728565d*tj) MOD 360d))*ddtor
  varpi=((102.9373481d)+(0.3225654d*tj))*ddtor
  ggeo=dlm-varpi
  lgeo=dlm+(1.915d*SIN(ggeo)+0.02d*SIN(2d*ggeo))*ddtor
  rgeo=(1.00014d)-0.01617d*COS(ggeo)-0.00014d*COS(2d*ggeo)
  IF debug THEN  PRINT,'%HEOMAT:dlm[deg],lgeo[deg],rgeo[AU]:',(dlm/ddtor) MOD 360d,(lgeo/ddtor) MOD 360d,rgeo
  lon=lgeo+!dpi
 
  ;HAE to GSE:
  haegse=GPOLROT(0.0d,0.0d,lon)
  IF insys EQ 'HAE' AND outsys EQ 'GSE' THEN RETURN,haegse

  ;GSE to HAE:
  gsehae=MARRMULO(haegse,/inv1)
  IF insys EQ 'GSE' AND outsys EQ 'HAE' THEN RETURN,gsehae

  IF hset THEN BEGIN
  ;HAE to HEE:
  haehee=GPOLROT(0.0d,0.0d,lon+!dpi)
  gsehee=MARRMULO(gsehae,haehee)
  IF insys EQ 'HEE' THEN BEGIN
    CASE outsys OF
      'HAE':RETURN,MARRMULO(haehee,/inv1)
      'GSE':RETURN,MARRMULO(gsehee,/inv1)
      ELSE:
    ENDCASE
  ENDIF
  IF outsys EQ 'HEE' THEN BEGIN
    CASE insys OF
      'HAE':RETURN,haehee
      'GSE':RETURN,gsehee
      ELSE:
     ENDCASE
  ENDIF


  ; heliographic:  
  ; there is a 5% difference here with respect to M.Hapgood's test values
  ; unresolved
  ; ecliptic solar rot pole   ;(Seidelmann, T15.4)
  ;omegapole=((75.766666667d)+1.4d*t0)*ddtor ; deg asc.node.
  ; new value compared to (Hapgood1) which is used here:
  ;omegapole=((73.6667d)+0.013958d*(mjdfrac+3242.0d)/365.25)*ddtor ; deg asc.node
  omegapole=((75.76d)+1.397d*tj)*ddtor
  ipole=7.25d*ddtor   ;deg

  aberr=20d/3600d*ddtor
  londif=(lon-aberr-omegapole) MOD (2*!dpi)
  
  thetacentral=ATAN(COS(ipole)*TAN(londif))
  
  ; put it into opposite quadrant:
  qindex=WHERE(ABS(londif-thetacentral) LT pi2)

  IF qindex(0) NE -1 THEN thetacentral(qindex)=thetacentral(qindex)+!dpi
  IF debug THEN   PRINT,'%HEOMAT:londif[deg],thetacen[deg],omegapole[deg]:', $
    londif/ddtor,thetacentral/ddtor,omegapole/ddtor
  
  haeheeq=GPOLROT(omegapole,ipole,thetacentral) 
  haehgd=GPOLROT(omegapole,ipole,0.0d)
  hgdheeq=MARRMULO(haehgd,haeheeq,/inv1)

  ; equiv to:
  ;haeheeq1=MARRMULO(GPOLROT(0.0,0.0,omegapole),MARRMULO(GPOLROT(0.0,ipole,0.0),$
  ;  GPOLROT(0.0,0.0,thetacentral)))
 
  gseheeq=MARRMULO(gsehae,haeheeq)
  IF insys EQ 'HEEQ' THEN BEGIN
    CASE outsys OF
      'HAE':RETURN,MARRMULO(haeheeq,/inv1)
      'GSE':RETURN,MARRMULO(gseheeq,/inv1)
      'HEE':RETURN,MARRMULO(haeheeq,haehee,/inv1)
      'HGD':RETURN,MARRMULO(hgdheeq,/inv1)
      ELSE:
     ENDCASE
  ENDIF
  IF outsys EQ 'HEEQ' THEN BEGIN
    CASE insys OF
      'HAE':RETURN,haeheeq
      'GSE':RETURN,gseheeq
      'HEE':RETURN,MARRMULO(haehee,haeheeq,/inv1)
      'HGD':RETURN,hgdheeq
      ELSE:
     ENDCASE
  ENDIF

  gsehgd=MARRMULO(gsehae,haehgd)
  
  IF insys EQ 'HGD' THEN BEGIN
    CASE outsys OF
      'HAE':RETURN,MARRMULO(haehgd,/inv1)
      'GSE':RETURN,MARRMULO(gsehgd,/inv1)
      'HEE':RETURN,MARRMULO(haehgd,haehee,/inv1)
      ELSE:
     ENDCASE
  ENDIF
  IF outsys EQ 'HGD' THEN BEGIN
    CASE insys OF
      'HAE':RETURN,haehgd
      'GSE':RETURN,gsehgd
      'HEE':RETURN,MARRMULO(haehee,haehgd,/inv1)
      ELSE:
     ENDCASE
  ENDIF

  ; HGC:

  geihgc=GPOLROT(286.13d*ddtor+pi2,pi2-63.87d*ddtor,((84.1d)+14.1844d*dj)*ddtor) 
  j1854=2398220d
  haehgc=GPOLROT(omegapole,ipole,(dj+jd2000-j1854)/25.38d*!dpi*2) 
  gsehgc=MARRMULO(gsehae,haehgc)
  IF insys EQ 'HGC' THEN BEGIN
    CASE outsys OF
      'GEI':RETURN,MARRMULO(geihgc,/inv1)
      'HAE':RETURN,MARRMULO(haehgc,/inv1)
      'GSE':RETURN,MARRMULO(gsehgc,/inv1)
      'HEE':RETURN,MARRMULO(haehgc,haehee,/inv1)
      'HEEQ':RETURN,MARRMULO(haehgc,haeheeq,/inv1)
      'HGD':RETURN,MARRMULO(haehgc,haehgd,/inv1)
      ELSE:
     ENDCASE
  ENDIF
  IF outsys EQ 'HGC' THEN BEGIN
    CASE insys OF
      'GEI':RETURN,geihgc
      'HAE':RETURN,haehgc
      'GSE':RETURN,gsehgc
      'HEE':RETURN,MARRMULO(haehee,haehgc,/inv1)
      'HEEQ':RETURN,MARRMULO(haeheeq,haehgc,/inv1)
      'HGD':RETURN,MARRMULO(haehgd,haehgc,/inv1)
       ELSE:
     ENDCASE
  ENDIF

   
  
  

  ENDIF ; hset


  ; EARTH systems:

  IF geoin OR STRPOS(outsys,'H') LT 0 THEN BEGIN

  ; earth obliquity (J2000):

  epstrue=EPSILON( tj,/true,/deg); deg
  IF debug THEN PRINT,'%HEOMAT:EpsTrue:',epstrue

  geigse=MARRMULO(GPOLROT(0.0d,epstrue*ddtor,0.0d),GPOLROT(0.0d,0.0d,lon))
  IF insys EQ 'GEI' THEN BEGIN
    CASE outsys OF
     'GSE': RETURN,geigse
     'HAE': RETURN,MARRMULO(geigse,gsehae)
     'HEE': RETURN,MARRMULO(geigse,gsehee)
     'HEEQ': RETURN,MARRMULO(geigse,gseheeq)
     'HGD': RETURN,MARRMULO(geigse,gsehgd)
     ELSE:
    ENDCASE 
  ENDIF
  IF outsys EQ 'GEI' THEN BEGIN
    CASE insys OF
     'GSE': RETURN,MARRMULO(geigse,/inv1)
     'HAE': RETURN,MARRMULO(haegse,geigse,/inv2)
     'HEE': RETURN,MARRMULO(gsehee,geigse,/inv1,/inv2)
     'HEEQ': RETURN,MARRMULO(gseheeq,geigse,/inv1,/inv2)
     'HGD': RETURN,MARRMULO(gsehgd,geigse,/inv1,/inv2)
     ELSE:
    ENDCASE 
  ENDIF

   ; GMT angle (Hapgood1)
;  JULIAN_GREG,dj+jd2000,year,month,day,hour,minute,second,/break
;  uthour=(hour+(minute/60d)+(second/3600d))
;  gmtheta0=(100.461d)+(36000.77d)*tj+15.04107*uthour ;deg

  gmtheta=GMST(dj+jd2000) ;deg
  IF debug THEN PRINT,'%HEOMAT:Gmst:',gmtheta

  geogei=GPOLROT(0.0d,0.0d,-gmtheta*ddtor)
  gsegeo=MARRMULO(geigse,geogei,/inv1,/inv2)
  IF insys EQ 'GEO' THEN BEGIN
   CASE outsys OF
     'GEI': RETURN,geogei
     'GSE': RETURN,MARRMULO(gsegeo,/inv1)
     'HAE': RETURN,MARRMULO(gsegeo,gsehae,/inv1)
     'HEE': RETURN,MARRMULO(gsegeo,gsehee,/inv1)
     'HEEQ': RETURN,MARRMULO(gsegeo,gseheeq,/inv1)
     'HGC': RETURN,MARRMULO(gsegeo,gsehgc,/inv1)
     'HGD': RETURN,MARRMULO(gsegeo,gsehgd,/inv1)
     ELSE:
   ENDCASE
  ENDIF
  IF outsys EQ 'GEO' THEN BEGIN
   CASE insys OF
     'GEI': RETURN,MARRMULO(geogei,/inv1)
     'GSE': RETURN,gsegeo
     'HAE': RETURN,MARRMULO(gsehae,gsegeo,/inv1)
     'HEE': RETURN,MARRMULO(gsehee,gsegeo,/inv1)
     'HEEQ': RETURN,MARRMULO(gseheeq,gsegeo,/inv1)
     'HGC': RETURN,MARRMULO(gsehgc,gsegeo,/inv1)
     'HGD': RETURN,MARRMULO(gsehgd,gsegeo,/inv1)
     ELSE:
   ENDCASE
  ENDIF

  ; GSM system:
  ; earth magnetic axis (Hapgood1)
  ; there is a small correction in (Hapgood2) but
  ; did not check how to implement this.
 
  londip=((288.44d) - 0.04236d*tj*100d)*ddtor
  latdip=((79.53d) + 0.03556d*tj*100d)*ddtor
  dipgeo=PHTHRV([[londip],[(!dpi/2d)-latdip],[DBLARR(nelin)+1.0]])
  
 
  qe=MARRMULO(dipgeo,gsegeo,/inv2,/vec1)
  psim=ATAN(qe(*,1)/qe(*,2))
  gsegsm=GPOLROT(0.0d,-psim,0.0d)
  IF debug THEN PRINT,'HEOMAT:londip,latdip[deg],psidip[deg]:',londip/ddtor,latdip/ddtor,psim/ddtor
  ; dipole tilt angle:
  mudip=ATAN(qe(*,0)/SQRT(qe(*,1)*qe(*,1)+qe(*,2)*qe(*,2)))
  IF debug THEN PRINT,'HEOMAT:mudip[deg]',mudip/ddtor

  IF insys EQ 'GSM' THEN BEGIN
   CASE outsys OF
     'GEI': RETURN,MARRMULO(gsegsm,geigse,/inv1,/inv2)
     'GEO': RETURN,MARRMULO(gsegsm,gsegeo,/inv1,/inv2)
     'GSE': RETURN,MARRMULO(gsegsm,/inv1)
     'HAE': RETURN,MARRMULO(gsegsm,gsehae,/inv1)
     'HEE': RETURN,MARRMULO(gsegsm,gsehee,/inv1)
     'HEEQ': RETURN,MARRMULO(gsegsm,gseheeq,/inv1)
     'HGC': RETURN,MARRMULO(gsegsm,gsehgc,/inv1)
     'HGD': RETURN,MARRMULO(gsegsm,gsehgd,/inv1)
     ELSE:
   ENDCASE
  ENDIF
  IF outsys EQ 'GSM' THEN BEGIN
   CASE insys OF
     'GEI': RETURN,MARRMULO(geigse,gsegsm)
     'GEO': RETURN,MARRMULO(gsegeo,gsegsm,/inv1)
     'GSE': RETURN,gsegsm
     'HAE': RETURN,MARRMULO(gsehae,gsegsm,/inv1)
     'HEE': RETURN,MARRMULO(gsehee,gsegsm,/inv1)
     'HEEQ': RETURN,MARRMULO(gseheeq,gsegsm,/inv1)
     'HGC': RETURN,MARRMULO(gsehgc,gsegsm,/inv1)
     'HGD': RETURN,MARRMULO(gsehgd,gsegsm,/inv1)
     ELSE:
   ENDCASE
  ENDIF

  ; SM solar earth magnetospheric
  gsmsm=GPOLROT(pi2,mudip,-pi2)
  gsesm=MARRMULO(gsegsm,gsmsm)

  IF insys EQ 'SMA' THEN BEGIN
   CASE outsys OF
     'GSM': RETURN,MARRMULO(gsmsm,/inv1)
     'GEI': RETURN,MARRMULO(gsesm,geigse,/inv1,/inv2)
     'GEO': RETURN,MARRMULO(gsesm,gsegeo,/inv1)
     'GSE': RETURN,MARRMULO(gsesm,/inv1)
     'HAE': RETURN,MARRMULO(gsesm,gsehae,/inv1)
     'HEE': RETURN,MARRMULO(gsesm,gsehee,/inv1)
     'HEEQ': RETURN,MARRMULO(gsesm,gseheeq,/inv1)
     'HGC': RETURN,MARRMULO(gsesm,gsehgc,/inv1)
     'HGD': RETURN,MARRMULO(gsesm,gsehgd,/inv1)
     ELSE:
   ENDCASE
  ENDIF
  IF outsys EQ 'SMA' THEN BEGIN
   CASE insys OF
     'GSM': RETURN,gsmsm
     'GEI': RETURN,MARRMULO(geigse,gsesm)
     'GEO': RETURN,MARRMULO(gsegeo,gsesm,/inv1)
     'GSE': RETURN,gsesm
     'HAE': RETURN,MARRMULO(gsehae,gsesm,/inv1)
     'HEE': RETURN,MARRMULO(gsehee,gsesm,/inv1)
     'HEEQ': RETURN,MARRMULO(gseheeq,gsesm,/inv1)
     'HGC': RETURN,MARRMULO(gsehgc,gsesm,/inv1)
     'HGD': RETURN,MARRMULO(gsehgd,gsesm,/inv1)
     ELSE:
   ENDCASE
  ENDIF

 ; MAG geomagnetic
  IF insys EQ 'MAG' OR outsys EQ 'MAG' THEN BEGIN
   geomag=GPOLROT(londip+pi2,pi2-latdip,-pi2)
   gsemag=MARRMULO(gsegeo,geomag)
   IF insys EQ 'MAG' THEN BEGIN
    CASE outsys OF
     'SMA': RETURN,MARRMULO(gsemag,gsesm,/inv1)
     'GSM': RETURN,MARRMULO(gsemag,gsegsm,/inv1)
     'GEI': RETURN,MARRMULO(gsemag,geigse,/inv1,/inv2)
     'GEO': RETURN,MARRMULO(gsemag,gsegeo,/inv1)
     'GSE': RETURN,MARRMULO(gsemag,/inv1)
     'HAE': RETURN,MARRMULO(gsemag,gsehae,/inv1)
     'HEE': RETURN,MARRMULO(gsemag,gsehee,/inv1)
     'HEEQ': RETURN,MARRMULO(gsemag,gseheeq,/inv1)
     'HGC': RETURN,MARRMULO(gsemag,gsehgc,/inv1)
     'HGD': RETURN,MARRMULO(gsemag,gsehgd,/inv1)
     ELSE:
   ENDCASE
   ENDIF
   IF outsys EQ 'MAG' THEN BEGIN
    CASE insys OF
     'SMA': RETURN,MARRMULO(gsesm,gsemag,/inv1)
     'GSM': RETURN,MARRMULO(gsegsm,gsemag,/inv1)
     'GEI': RETURN,MARRMULO(geigse,gsemag)
     'GEO': RETURN,MARRMULO(gsegeo,gsemag,/inv1)
     'GSE': RETURN,gsemag
     'HAE': RETURN,MARRMULO(gsehae,gsemag,/inv1)
     'HEE': RETURN,MARRMULO(gsehee,gsemag,/inv1)
     'HEEQ': RETURN,MARRMULO(gseheeq,gsemag,/inv1)
     'HGC': RETURN,MARRMULO(gsehgc,gsemag,/inv1)
     'HGD': RETURN,MARRMULO(gsehgd,gsemag,/inv1)
     ELSE:
   ENDCASE
  ENDIF

  ENDIF ; MAG

  ENDIF ; Earth systems

  ;HGRTN
  IF STRPOS(insys,'RTN') GT 0 OR STRPOS(outsys,'RTN') GT 0 THEN BEGIN
   
   ; we need heliocentric vectors to s/c and solar pole:
   IF geoin OR nelvec LE 0 THEN BEGIN
     ;earth pointing or position vector in sol cent ecl:
     IF nelvec GT 0 THEN au=1.4959787d8 ELSE au=1.0d ;km
     evec=DBLARR(nelin,3)
     evec(*,0)=COS(lon+!dpi)*au
     evec(*,1)=SIN(lon+!dpi)*au
;     ; rotate into heeq:
;     evec=MARRMULO(evec,HEOMAT(tj,insys='HAE',outsys='HEEQ'),/vec1)
   ENDIF

   IF nelvec GT 0 THEN BEGIN
     ;transform posvector to HAE system
     IF NOT KEYWORD_SET(possys) THEN BEGIN
      IF insys EQ 'HGRTN' THEN possys='HGD' $
       ELSE IF insys EQ 'HERTN' THEN possys='HGC' $
       ELSE possys=insys
     ENDIF
     ; calculate inheeq matrix by recursive call:
     inout=HEOMAT(tj,insys=possys,outsys='HAE')
     rvec=MARRMULO(posvector,inout,/vec1)
     ; for geographic input system add earth position:
     IF geoin THEN rvec=rvec+evec
   ENDIF ELSE rvec=evec

   IF debug THEN PRINT,'HEOMAT: POSVECHAE:',TRANSPOSE(rvec)
   ;solar polvec in hae
   polvec=DBLARR(nelin,3)
   ct=COS(ipole)
   st=SIN(ipole)
 
   lonpol=omegapole-pi2
   polvec(*,0)=COS(lonpol)*st
   polvec(*,1)=SIN(lonpol)*st
   polvec(*,2)=ct


   ; define HGRTN pole by RXT=RX(NXR)
   tvec=VECPRD(polvec,rvec)
   nvec=VECPRD(rvec,tvec)
   ; get polar angles for nvec and rvec:
   nphi=ATAN(nvec(*,1),nvec(*,0))
   nsq=nvec*nvec
   nth=ACOS(nvec(*,2)/SQRT(TOTAL(nsq,2)))
   rphi=ATAN(rvec(*,1),rvec(*,0))
 
   ; HAE to HGRTN 
   lonnode=nphi+pi2
   haehgrtn=GPOLROT(lonnode,nth,rphi-lonnode)
   ; GSE to HGRTN
   gsehgrtn=MARRMULO(gsehae,haehgrtn)
   IF insys EQ 'HGRTN' THEN BEGIN
    CASE outsys OF
     'MAG': RETURN,MARRMULO(gsehgrtn,gsemag,/inv1)
     'SMA': RETURN,MARRMULO(gsehgrtn,gsesm,/inv1)
     'GSM': RETURN,MARRMULO(gsehgrtn,gsegsm,/inv1)
     'GEI': RETURN,MARRMULO(gsehgrtn,geigse,/inv1,/inv2)
     'GEO': RETURN,MARRMULO(gsehgrtn,gsegeo,/inv1)
     'GSE': RETURN,MARRMULO(gsehgrtn,/inv1)
     'HAE': RETURN,MARRMULO(gsehgrtn,gsehae,/inv1)
     'HEE': RETURN,MARRMULO(gsehgrtn,gsehee,/inv1)
     'HEEQ': RETURN,MARRMULO(gsehgrtn,gseheeq,/inv1)
     'HGC': RETURN,MARRMULO(gsehgrtn,gsehgc,/inv1)
     'HGD': RETURN,MARRMULO(gsehgrtn,gsehgd,/inv1)
      ELSE:
   ENDCASE
   ENDIF
   IF outsys EQ 'HGRTN' THEN BEGIN
    CASE insys OF
     'MAG': RETURN,MARRMULO(gsemag,gsehgrtn,/inv1)
     'SMA': RETURN,MARRMULO(gsesm,gsehgrtn,/inv1)
     'GSM': RETURN,MARRMULO(gsegsm,gsehgrtn,/inv1)
     'GEI': RETURN,MARRMULO(geigse,gsehgrtn)
     'GEO': RETURN,MARRMULO(gsegeo,gsehgrtn,/inv1)
     'GSE': RETURN,gsehgrtn
     'HAE': RETURN,MARRMULO(gsehae,gsehgrtn,/inv1)
     'HEE': RETURN,MARRMULO(gsehee,gsehgrtn,/inv1)
     'HEEQ': RETURN,MARRMULO(gseheeq,gsehgrtn,/inv1)
     'HGC': RETURN,MARRMULO(gsehgc,gsehgrtn,/inv1)
     'HGD': RETURN,MARRMULO(gsehgd,gsehgrtn,/inv1)
     ELSE:
   ENDCASE
  ENDIF

   ; define HERTN pole by RXT=RX(NXR)

   ;ecliptic North:
   polvec=DBLARR(nelin,3)
   polvec(*,2)=1.0

   tvec=VECPRD(polvec,rvec)
   nvec=VECPRD(rvec,tvec)
   ; get polar angles for nvec and rvec:
   nphi=ATAN(nvec(*,1),nvec(*,0))
   nsq=nvec*nvec
   nth=ACOS(nvec(*,2)/SQRT(TOTAL(nsq,2)))
   rphi=ATAN(rvec(*,1),rvec(*,0))
 
   ; HAE to HERTN 
   lonnode=nphi+pi2
   haehertn=GPOLROT(lonnode,nth,rphi-lonnode)
   ; GSE to HERTN
   gsehertn=MARRMULO(gsehae,haehertn)
   IF insys EQ 'HERTN' THEN BEGIN
    CASE outsys OF
     'MAG': RETURN,MARRMULO(gsehertn,gsemag,/inv1)
     'SMA': RETURN,MARRMULO(gsehertn,gsesm,/inv1)
     'GSM': RETURN,MARRMULO(gsehertn,gsegsm,/inv1)
     'GEI': RETURN,MARRMULO(gsehertn,geigse,/inv1,/inv2)
     'GEO': RETURN,MARRMULO(gsehertn,gsegeo,/inv1)
     'GSE': RETURN,MARRMULO(gsehertn,/inv1)
     'HAE': RETURN,MARRMULO(gsehertn,gsehae,/inv1)
     'HEE': RETURN,MARRMULO(gsehertn,gsehee,/inv1)
     'HEEQ': RETURN,MARRMULO(gsehertn,gseheeq,/inv1)
     'HGC': RETURN,MARRMULO(gsehertn,gsehgc,/inv1)
     'HGD': RETURN,MARRMULO(gsehertn,gsehgd,/inv1)
     'HGRTN': RETURN,MARRMULO(gsehertn,gsehgrtn,/inv1)
      ELSE:
   ENDCASE
   ENDIF
   IF outsys EQ 'HERTN' THEN BEGIN
    CASE insys OF
     'MAG': RETURN,MARRMULO(gsemag,gsehertn,/inv1)
     'SMA': RETURN,MARRMULO(gsesm,gsehertn,/inv1)
     'GSM': RETURN,MARRMULO(gsegsm,gsehertn,/inv1)
     'GEI': RETURN,MARRMULO(geigse,gsehertn)
     'GEO': RETURN,MARRMULO(gsegeo,gsehertn,/inv1)
     'GSE': RETURN,gsehertn
     'HAE': RETURN,MARRMULO(gsehae,gsehertn,/inv1)
     'HEE': RETURN,MARRMULO(gsehee,gsehertn,/inv1)
     'HEEQ': RETURN,MARRMULO(gseheeq,gsehertn,/inv1)
     'HGC': RETURN,MARRMULO(gsehgc,gsehertn,/inv1)
     'HGD': RETURN,MARRMULO(gsehgd,gsehertn,/inv1)
     'HGRTN': RETURN,MARRMULO(gsehgrtn,gsehertn,/inv1)
     ELSE:
   ENDCASE
  ENDIF


  ENDIF ; RTN

 
 MESSAGE,'Transformation case undefined'
 RETURN,-1
 END


