;####################TAKEN FROM F77 ROUTINE PLANETAP.F BY J.L. Simon
;     Object :
;     Computation of approximative ephemerides of the 8 major planets.
;     - longitude, latitude, radius vector.
;     - rectangular heliocentric coordinates and velocities.
;     Reference frame : Equinox and ecliptic J2000 (JD2451545.0).
;
;     Reference :
;     Astron. Astrophys. 282, 663 (1994).
;
;     Authors :
;     J.L. Simon, P. Bretagnon, J. Chapront, M. Chapront-Touze,
;     G. Francou, J. Laskar (Bureau des Longitudes, Paris, France).
;
;     Input parameters :
;     dj        Julian date tdb (double real).
;     iplain    planet index (integer).
;               ipla=1 Mercury                  ipla=5 Jupiter
;               ipla=2 Venus                    ipla=6 Saturn
;               ipla=3 Earth-Moon Barycenter    ipla=7 Uranus
;               ipla=4 Mars                     ipla=8 Neptune
;               ipla=10 Moon                   ipla=12 Earth
;
;     Keywords: date rotate into ecliptic of date
;               order -2 : use 1st order approx from paper, no dist.
;               order -1 : use full 1st order terms and dist
;                      1 : use full 1st order terms, no dist
;                      2 : use full 2nd order terms, no dist
;                      3 : use full 2nd order terms and dist
;                      4 : use mean, not true anomaly
;
;     Output parameters :
;     dlong     longitude in radians (double real).
;     dlat      latitude  in radians (double real).
;     rvec      radius vector in au  (double real).
;     xyz(6)    rectangular heliocentric coordinates (double real).
;               xyz(1) : X in au     xyz(4) : X' in au/day.
;               xyz(2) : Y in au     xyz(5) : Y' in au/day.
;               xyz(3) : Z in au     xyz(6) : Z' in au/day.
;     ierr      error index (integer)
;               ierr=0 no error.
;               ierr=1 error in planet index ipla.
;               ierr=2 error in Kepler equation.
;               ierr=3 date outside of the interval (1000-3000).
;

 PRO PLANETAP,dj,iplain,dlong,dlat,rvec,pos,vvel,ierr,date=date,order=order
 
 IF NOT KEYWORD_SET(order) THEN order=3

 dateset=KEYWORD_SET(date)

;     Time parameters.
;
         t=(dj-2451545.d0)/365250.d0
         t2=t*t
  

 counts=N_ELEMENTS(dj)

 xyz=DBLARR(counts,6) 
 ; mass ratio sun/planet IAU76:
 ;  mer,ven,emb,jup,sat,ura,nep
 amas=[6023600.d0,408523.5d0,328900.5d0,3098710.d0, $
          1047.355d0,  3498.5d0,  22869.d0,  19314.d0]
 sdrad=0.4848136811095360d-5 ; arcsec to radian
 dpi=6.283185307179586d0
 dtor=!dpi/180.0d
 ierr=0
 
 IF iplain GE 10 THEN ipla=2 ELSE ipla=iplain-1

; 
 PLANEDAT,a,dlm,e,pi,dinc,omega,kp,ca,sa,kq,cl,sl

 IF order LE -2 THEN BEGIN
  PLANEDAT1,ipla,t*10.0d,da,dl,de,dp,di,dom
 ENDIF ELSE BEGIN

;     Computation of the mean elements.
;
         da=a(1-1,ipla)+t*a(2-1,ipla)
         dl=(3600.d0*dlm(1-1,ipla)+t*dlm(2-1,ipla))*sdrad
         de=e(1-1,ipla)+t*e(2-1,ipla)
         dp=(3600.d0*pi(1-1,ipla)+t*pi(2-1,ipla))*sdrad
         di=(3600.d0*dinc(1-1,ipla)+t*dinc(2-1,ipla))*sdrad
         dom=(3600.d0*omega(1-1,ipla)+t*omega(2-1,ipla))*sdrad
	 
	 IF order GE 2 THEN BEGIN
          da=da+t2*a(3-1,ipla)
          dl=dl+(t2*dlm(3-1,ipla))*sdrad
          de=de+t2*e(3-1,ipla)
          dp=dp+(t2*pi(3-1,ipla))*sdrad
          di=di+(t2*dinc(3-1,ipla))*sdrad
          dom=dom+(t2*omega(3-1,ipla))*sdrad
	 ENDIF
  ENDELSE
  dp=dp MOD dpi
  dom=dom MOD dpi

;print,dlm(1-1,ipla),t*dlm(2-1,ipla),t2*dlm(3-1,ipla)
;print,(3600.d0*dlm(1-1,ipla)+t*dlm(2-1,ipla)+t2*dlm(3-1,ipla))/3600
;print,[da,dl/dtor,de,dp/dtor,di/dtor,dom/dtor]
;
;
;     Computation of the trigonometric terms.
;
;
         IF (order GE 3 AND order LE 5) OR order EQ -1 OR order LE -3 THEN BEGIN
          dmu=0.35953620*t
          FOR j=1,8 DO BEGIN
            arga=kp(j-1,ipla)*dmu
            argl=kq(j-1,ipla)*dmu
            da=da+(ca(j-1,ipla)*cos(arga)+sa(j-1,ipla)*sin(arga))*1.d-7
            dl=dl+(cl(j-1,ipla)*cos(argl)+sl(j-1,ipla)*sin(argl))*1.d-7
          ENDFOR

          arga=kp(9-1,ipla)*dmu
          da=da+t*(ca(9-1,ipla)*cos(arga)+sa(9-1,ipla)*sin(arga))*1.d-7
          FOR j=9,10 DO BEGIN
            argl=kq(j-1,ipla)*dmu
            dl=dl+t*(cl(j-1,ipla)*cos(argl)+sl(j-1,ipla)*sin(argl))*1.d-7
          ENDFOR
         ENDIF
         dl=dl MOD dpi
	 
	 ; approx by low precision solar formulas:
	 IF order EQ -4 THEN BEGIN
	  mano=dl-dp
	  dl=dl+(1.915d*SIN(mano)+0.02d*SIN(2*mano))*!dtor
	  da=((1.00014d)-0.01671d*COS(mano)-0.00014d*COS(2*mano))/SQRT(1-de*de)
	 ENDIF
;
;     print,[da,dl/dtor,de,dp/dtor,di/dtor,dom/dtor]
;
;     Computation of the rectangular coordinates and velocities.
;
         dmas=1.d0/amas(ipla)
         ierr=0
         xyz=ORBIT(da,dl,de,dp,di,dom,dmas,nokep=(ABS(order) EQ 4))

	 pos=xyz(*,0:2)
	 vvel=xyz(*,3:5)

;From Jean-Louis.Simon@bdl.fr Tue Oct 17 15:55 BST 2000	 
;b) We compute the heliocentric longitude (L), latitude (B) and distance (R)
;of the Earth from the EMB values by adding the corrections  :

;Delta L = 3.136d-5 sin D
;Delta B = 0
;Delta R = 3.0837d-5 cos D 
;where the units are radian for L and ua for R

;Using the expression of the Delaunay argument D given in our paper, the precedent
;corrections may be writen upon the form :

;Delta L = 3.136d-5 cos(3.62767+77713.77147 T)
;Delta B = 0
;Delta R = 3.0837d-5 cos(5.19847+77713.77147 T)
;where T is measured in thousands of Julian years from J2000

; i.e. D=297.85039^o+1236.8531'' T
;      L=6.468''sinD  R=4613[km]cosD


     IF iplain GE 10 THEN BEGIN
         posemb=pos
         pang=VPHTHR(pos)
	 td=(5.19847d)+77713.77147d*t
	 dphi=(3.136d-5)*SIN(td)
	 dr=(3.0837d-5)*COS(td)
         pang(*,0)=pang(*,0)+dphi
         pang(*,2)=pang(*,2)+dr
         pos=PHTHRV(pang)  
	 
	 
	 ;velocity:
	 tddot=77713.77147d/365250d ; per day
	 drdot=-(3.0837d-5)*SIN(td)*tddot
	 dphidot=(3.136d-5)*COS(td)*tddot
	 r=SQRT(TOTAL(pos*pos,2))
	 rdot=SQRT(TOTAL(vvel*vvel,2))
	 x=pos(*,0)
	 y=pos(*,1)
	 xdot=vvel(*,0)
	 ydot=vvel(*,1)
	 cosdp=COS(dphi)
	 sindp=SIN(dphi)
	 a=(1+dr/r)
	 b=x*cosdp-y*sindp
	 c=y*cosdp+x*sindp
	 adot=drdot/r-dr*rdot/(r*r)
	 bdot=(xdot-y*dphidot)*cosdp-(x*dphidot+ydot)*sindp
	 cdot=(xdot-y*dphidot)*sindp+(ydot+x*dphidot)*cosdp
	 
 	 vearth=vvel
         vearth(*,0)=adot*b+a*bdot
         vearth(*,1)=adot*c+a*cdot
	 
	 ; simpler formula:
	 
	 posdif=pos-posemb
         period=445267.11d/360d/36525d ; rotations/day
         veldif=TRANSPOSE([-posdif(*,1),posdif(*,0),posdif(*,0)*0.0d])*2d*!dpi*period
	 vearth=vvel+veldif

	 IF iplain EQ 10 THEN BEGIN ; calculate position of moon from Earth pos
	    emrat = 81.30056d
            pos=(1+emrat)*posemb-emrat*pos
	    vvel=(1+emrat)*vvel-emrat*vearth
	 ENDIF ELSE vvel=vearth

 
     ENDIF

	 
; now rotate into ecliptic of date:
         IF dateset THEN BEGIN
	    tj=(dj-2451545.0d)/36525d
            prec=PRECESS(tj*0.0d,tj,order=3,ang=angarr)
	    eps=EPSILON(tj,order=order)
            nut=NUTATION(tj,dpde,order=order)

	    IF date EQ 1 THEN BEGIN
	     FOR i=0,counts-1 DO BEGIN
	       pos(i,*)=pos(i,*)#REFORM(prec(i,*,*))
	       vvel(i,*)=vvel(i,*)#REFORM(prec(i,*,*))
	     ENDFOR
	    ENDIF
	 ENDIF
         pang=VPHTHR(pos)
         dlong=pang(*,0) MOD (2*!dpi)
         dlat=pang(*,1)
         rvec=pang(*,2)



 RETURN
 END
